
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include "w4ais.h"


/* Sets everything up for calling waissearch and executes it.
** Here, waissearch is expected to return a source description or
** file.
*/

void descwaiss()
{
	int i, j, k;
	char *argline[MAXARGS], word[MAXSTRLEN];
	int pipe0[2], pipe1[2];
	FILE *fp0, *fp1;

	argline[0] = waissearchbin;

	argline[1] = (char *) mystrdup("-h");
	argline[2] = host;
	argline[3] = (char *) mystrdup("-p");
	argline[4] = port;
	argline[5] = (char *) mystrdup("-d");
	argline[6] = source;
	argline[7] = (char *) mystrdup("-m");
	argline[8] = maxhits;

	for (i = 0, k = 9;;) {
		for (j = 0; keywords[i] != '\0' &&
		keywords[i] != ' ' && j < MAXSTRLEN; i++)
			word[j++] = keywords[i];
		word[j] = '\0';

		argline[k++] = (char *) mystrdup(word);

		if (keywords[i] == '\0')
			break;
		else if (keywords[i] == ' ')
			i++;
	}
	argline[k] = (char *) 0;

	pipe(pipe0);
	pipe(pipe1);
	switch(fork()) {
	case -1:
		progerr("fork() call failed.");
	case 0:
		close(pipe0[1]);
		close(pipe1[0]);
		dup2(pipe0[0], 0);
		dup2(pipe1[1], 1);
		execv(waissearchbin, argline);
		progerr("execv() call failed.");
	default:
		close(pipe0[0]);
		close(pipe1[1]);
		fp0 = fdopen(pipe1[0], "r");
		fp1 = fdopen(pipe0[1], "w");
		desc_waissearch(fp0, fp1);
	}
}

/* Reads lines returned from waissearch and grabs a source
** description or a file.
*/

void desc_waissearch(fp, fp2)
	FILE *fp;
	FILE *fp2;
{
	int i, end, srcline = 0, found, isdesc, gotmatch;
	char *c, *d, buffer[MAXSTRLEN], filename[MAXSTRLEN],
	maintainer[MAXSTRLEN], cost[MAXSTRLEN], costunit[MAXSTRLEN];

	found = -1;
	gotmatch = 0;
	while (fgets(buffer, MAXSTRLEN, fp) != NULL) {
		if (gotmatch && !strchr(buffer, ':'))
			break;
		if (strstr(buffer, "NumberOfRecordsReturned")) {
			found = getnumber(buffer, 1);
			if (!found)
				return;
		}
		if (strstr(buffer, "Information on database") ||
		strstr(buffer, "Catalog for database") ||
		strstr(buffer, "Search produced no") ||
		strstr(buffer, "is not available"))
			return;
		if ((c = (char *) strstr(buffer, "lines:")) != NULL &&
		!gotmatch) {
			for (i = srcline = 0; buffer[i] != ':'; i++)
				if (isdigit(buffer[i]))
					srcline = (srcline * 10) +
					(buffer[i] - '0');
			strcpy(filename, getsfilename(buffer, srcline));
			if (docnum == srcline)
				gotmatch = 1;
		}
	}
	fprintf(fp2, "%d\n", srcline);
	fflush(fp2);
	fgets(buffer, MAXSTRLEN, fp);
	if (lstrstr(filename, ".src")) {
		isdesc = 1;
		end = 0;
		while (fgets(buffer, MAXSTRLEN, fp) != NULL && !end) {
			if ((c = (char *) strstr(buffer, ":ip-name"))
			!= NULL) {
				c += strlen(":ip-name");
				strcpy(host, getword(c, &i));
			}
			else if ((c = (char *) strstr(buffer, ":cost "))
			!= NULL) {
				c += strlen(":cost ");
				strcpy(cost, getword(c, &i));
			}
			else if ((c = (char *) strstr(buffer, ":cost-unit :"))
			!= NULL) {
				c += strlen(":cost-unit :");
				strcpy(costunit, getword(c, &i));
			}
			else if ((c = (char *) strstr(buffer, ":tcp-port"))
			!= NULL) {
				c += strlen(":tcp-port");
				strcpy(port, getword(c, &i));
			}
			else if ((c = (char *) strstr(buffer, ":database-name"))
			!= NULL) {
				c += strlen(":database-name");
				strcpy(source, getword(c, &i));
			}
			else if ((c = (char *) strstr(buffer, ":maintainer"))
			!= NULL ) {
				c += strlen(":maintainer");
				strcpy(maintainer, getword(c, &i));
			}
			else if ((c = (char *) strstr(buffer, ":description"))
			!= NULL) {
				strcpy(buffer, replace(buffer, "\\\"", "\'"));
				c = (char *) strchr(buffer, '\"');
				while (c == NULL) {
					fgets(buffer, MAXSTRLEN, fp);
					c = (char *) strchr(buffer, '\"');
				}
				c++;
				if (numchar(buffer, '\"') == 2 &&
				(d = (char *) strrchr(buffer, '\"'))) {
					*d = '\0';
					desclist = (struct source *)
					addsource(desclist, c);
					break;
				}
				desclist = (struct source *)
				addsource(desclist, c);
	  		 		while (fgets(buffer, MAXSTRLEN, fp) != NULL) {
					strcpy(buffer, replace(buffer, "\\\"",
					"\'"));
					if (NULL != (c = (char *) strchr(buffer, '\"'))) {
						*c = '\0';
						desclist = (struct source *)
						addsource(desclist, buffer);
						end = 1;
						break;
					}
					desclist = (struct source *)
					addsource(desclist, buffer);
				}
			}
		}
	}
	else {
		isdesc = 0;
		while (fgets(buffer, 40, fp) != NULL) {
			if (!strncmp(buffer, "View doc", 8))
				break;
			bodylist = (struct source *)
			addsource(bodylist, buffer);
		}
	}
	fprintf(fp2, "q\n");
	fflush(fp2);
	fprintf(fp2, "q\n");
	fflush(fp2);

	indexsources = 0;
	if (isdesc) {
		no_options = 1;
		printheader();
		printform();
		printf("<h2>Database name: %s</h2>\n<p>\n", source);
		printf("<b>Maintainer:</b> <a href=\"mailto:%s\">%s</a>\n<p>\n",
		maintainer, maintainer);
		printf("<b>Description:</b>\n<p>\n");
		printf("<pre>\n");
		while (desclist != NULL) {
			printf("%s", desclist->line);
			desclist = desclist->next;
		}
		printf("</pre>\n<p>\n");
		printf("<b>Cost:</b> %s<br>\n", cost);
		printf("<b>Cost-unit:</b> %s<br>\n", costunit);
		printf("<b>Host:</b> %s<br>\n", host);
		printf("<b>Port:</b> %s<p>\n", port);
		printfooter();
	}
	else {
		printf("Content-type: %s\n\n", getmime(filename));
		while (bodylist != NULL) {
			printf("%s", bodylist->line);
			bodylist = bodylist->next;
		}
	}
}
