
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include "w4ais.h"


/* Sets everything up for calling swish and executes it.
** Spits out version information if requested.
*/

void doswish()
{
    int i, j, k;
    char *argline[MAXARGS], word[MAXSTRLEN], sourcepath[MAXSTRLEN];
    int pipe0[2], pipe1[2];
    FILE *fp0;

    argline[0] = swishbin;

    if (version) {
        argline[1] = (char *) mystrdup("-V");
        argline[2] = (char *) 0;
    } else {
        sprintf(sourcepath, "%s%s%s", sourcedir,
        (sourcedir[strlen(sourcedir) - 1] == '/') ?
        "" : "/", source);
        argline[1] = (char *) mystrdup("-f");
        argline[2] = sourcepath;
        argline[3] = (char *) mystrdup("-m");
        argline[4] = maxhits;
        argline[5] = (char *) mystrdup("-w");

        for (i = 0, k = 6; k < MAXARGS;) {
            for (j = 0; keywords[i] != '\0' &&
            keywords[i] != ' ' && j < MAXSTRLEN; i++)
                word[j++] = keywords[i];
            word[j] = '\0';

            argline[k++] = (char *) mystrdup(word);

            if (keywords[i] == '\0')
                break;
            else if (keywords[i] == ' ')
                i++;
        }

        argline[k] = (char *) 0;
    }

    pipe(pipe0);
    pipe(pipe1);
    switch(fork()) {
    case -1:
        progerr("fork() call failed.");
    case 0:
        close(pipe0[1]);
        close(pipe1[0]);
        dup2(pipe0[0], 0);
        dup2(pipe1[1], 1);
        execv(swishbin, argline);
        progerr("execv() call failed.");
    default:
        close(pipe0[0]);
        close(pipe1[1]);
        fp0 = fdopen(pipe1[0], "r");
        read_from_swish(fp0);
        if (!version) {
            printheader();

            if (entrylist == NULL) {
                notfound();
                printform();
                return;
            }

            printf("<HR>Here is the result of your search using ");
            printf("%s ", promptstring);
            printf("<b>\"%s\"</b>:<p>\n", keywords);

            if (!strcmp(format, "brief")) {
                printf("<OL>\n");
                printBriefEntries(entrylist);
                printf("</OL>\n");
            } else if (!strcmp(format, "table")) {
                printf("<TABLE BORDER=1>\n");
                printf("<TR><TH>Rank</TH><TH>Score</TH><TH>Type</TH><TH>Size</TH><TH>Title</TH></TR>\n");
                printTableEntries(entrylist);
                printf("</TABLE>\n");
            } else {    /* must be standard */
                printf("<DL>\n");
                printentries(entrylist);
                printf("</DL>\n");
            }
            printform();
        }
    }
}


/* Reads returned lines from swish, grabs the relevant information,
** and sticks it all in the return information structure.
*/

void read_from_swish(fp)
    FILE *fp;
{
    int i, j, score, bytes;
    static char buffer[MAXSTRLEN], filename[MAXSTRLEN], title[MAXSTRLEN],
    type[MAXSTRLEN], icon[MAXSTRLEN];

    while (fgets(buffer, MAXSTRLEN, fp) != NULL) {
        if (version) {
            printf("%s version: %s\n", swishbin, buffer);
            break;
        }
        if (buffer[0] == '.' || buffer[0] == 'e')
            break;
        if (buffer[0] == '#' || buffer[0] == 's')
            continue;
        if (strstr(buffer, "no results"))
            return;

        for (i = score = 0; isdigit(buffer[i]); i++)
            score = (score * 10) + (buffer[i] - '0');

        for (i++, j = 0; buffer[i] != ' '; i++)
            filename[j++] = buffer[i];
        filename[j] = '\0';

        for (i += 2, j = 0; buffer[i] != '\"'; i++)
            title[j++] = buffer[i];
        title[j] = '\0';

        for (i += 2, bytes = 0; isdigit(buffer[i]); i++)
            bytes = (bytes * 10) + (buffer[i] - '0');

        strcpy(type, (char *) gettype(filename));
        if (use_icons)
            strcpy(icon, (char *) geticon(filename));
        else
            strcpy(icon, NO_ICON);

        entrylist = (struct entry *) addentry(entrylist,
            score, -1, bytes, filename, title, type, icon, -1);
    }
}
