
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include "w4ais.h"


/* Sets everything up for calling waisq and executes it.
** Spits out version information if requested.
*/

void dowaisq()
{
	int i, j, k;
	char *argline[MAXARGS], word[MAXSTRLEN];
	int pipe0[2], pipe1[2];
	FILE *fp0;

	argline[0] = waisqbin;

	if (version) {
		argline[1] = (char *) mystrdup("-V");
		argline[2] = (char *) 0;
	}
	else {
		argline[1] = (char *) mystrdup("-s");
		argline[2] = sourcedir;
		argline[3] = (char *) mystrdup("-f");
		argline[4] = (char *) mystrdup("-");
		argline[5] = (char *) mystrdup("-S");
		argline[6] = source;
		argline[7] = (char *) mystrdup("-m");
		argline[8] = maxhits;
		argline[9] = (char *) mystrdup("-g");

		for (i = 0, k = 10; k < MAXARGS;) {
			for (j = 0; keywords[i] != '\0' &&
			keywords[i] != ' ' && j < MAXSTRLEN; i++)
				word[j++] = keywords[i];
			word[j] = '\0';

			argline[k++] = (char *) mystrdup(word);

			if (keywords[i] == '\0')
				break;
			else if (keywords[i] == ' ')
				i++;
		}

		argline[k] = (char *) 0;
	}

	pipe(pipe0);
	pipe(pipe1);
	switch(fork()) {
	case -1:
		progerr("fork() call failed.");
	case 0:
		close(pipe0[1]);
		close(pipe1[0]);
		dup2(pipe0[0], 0);
		dup2(pipe1[1], 1);
		execv(waisqbin, argline);
		progerr("execv() call failed.");
	default:
		close(pipe0[0]);
		close(pipe1[1]);
		fp0 = fdopen(pipe1[0], "r");
		read_from_waisq(fp0);
		if (!version) {
			printheader();

			if (entrylist == NULL) {
				notfound();
				printform();
				return;
			}

			printf("<HR>Here is the result of your search using ");
			printf("%s ", promptstring);
			printf("<b>\"%s\"</b>:<p>\n", keywords);

			printf("<dl>\n");
			printentries(entrylist);
			printf("</dl>\n");
			printform();
		}
	}
}

/* Reads returned lines from waisq, grabs the relevant information,
** and sticks it all in the return information structure.
*/

void read_from_waisq(fp)
	FILE *fp;
{
	int score = 0, lines = 0, bytes = 0, skipline = 0;
	static char buffer[MAXSTRLEN], filename[MAXSTRLEN], title[MAXSTRLEN],
	type[MAXSTRLEN], icon[MAXSTRLEN];

	while (fgets(buffer, MAXSTRLEN, fp) != NULL) {
		if (strstr(buffer, "Information on database") ||
		strstr(buffer, "Catalog for database"))
			skipline = 1;
		if (version) {
			printf("%s\n", buffer);
			break;
		}
		if (strstr(buffer, "Search produced no"))
			return;
		if (strstr(buffer, ":score") && !skipline)
			score = getnumber(buffer, 1);
		if (strstr(buffer, ":original-local-id") && !skipline) {
			strcpy(filename, (char *) getqfilename(buffer));
			strcpy(type, (char *) gettype(filename));
			if (use_icons)
				strcpy(icon, (char *) geticon(filename));
			else
				strcpy(icon, NO_ICON);
			strcpy(title, (char *) gettitle(filename, -1));
		}
		if (strstr(buffer, ":number-of-lines") && !skipline)
			lines = getnumber(buffer, 1);
		if (strstr(buffer, ":number-of-bytes")) {
			if (skipline) {
				skipline = 0;
				continue;
			}
			bytes = getnumber(buffer, 1);
			entrylist = (struct entry *) addentry(entrylist,
			score, lines, bytes, filename, title, type, icon, -1);
		}
	}
}
