
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include "w4ais.h"


/* Sets everything up for calling waissearch and executes it.
** Spits out version information if requested.
*/

void dowaiss()
{
	int i, j, k;
	char *argline[MAXARGS], word[MAXSTRLEN];
	int pipe0[2], pipe1[2];
	FILE *fp0, *fp1;

	argline[0] = waissearchbin;

	if (version) {
		argline[1] = (char *) mystrdup("-v");
		argline[2] = (char *) 0;
	}
	else {
		argline[1] = (char *) mystrdup("-h");
		argline[2] = host;
		argline[3] = (char *) mystrdup("-p");
		argline[4] = port;
		argline[5] = (char *) mystrdup("-d");
		argline[6] = source;
		argline[7] = (char *) mystrdup("-m");
		argline[8] = maxhits;

		for (i = 0, k = 9;;) {
			for (j = 0; keywords[i] != '\0' &&
			keywords[i] != ' ' && j < MAXSTRLEN; i++)
				word[j++] = keywords[i];
			word[j] = '\0';

			argline[k++] = (char *) mystrdup(word);

			if (keywords[i] == '\0')
				break;
			else if (keywords[i] == ' ')
				i++;
		}

		argline[k] = (char *) 0;
	}

	pipe(pipe0);
	pipe(pipe1);
	switch(fork()) {
	case -1:
		progerr("fork() call failed.");
	case 0:
		close(pipe0[1]);
		close(pipe1[0]);
		dup2(pipe0[0], 0);
		dup2(pipe1[1], 1);
		execv(waissearchbin, argline);
		progerr("execv() call failed.");
	default:
		close(pipe0[0]);
		close(pipe1[1]);
		fp0 = fdopen(pipe1[0], "r");
		fp1 = fdopen(pipe0[1], "w");
		read_from_waissearch(fp0);
		if (!version) {
			fprintf(fp1, "q\n");
			fflush(fp1);
			fprintf(fp1, "q\n");
			fflush(fp1);

			printheader();

			if (entrylist == NULL) {
				notfound();
				printform();
				return;
			}

			printf("<HR>Here is the result of your search using ");
			printf("%s ", promptstring);
			printf("<b>\"%s\"</b>:<p>\n", keywords);

			printf("<dl>\n");
			printentries(entrylist);
			printf("</dl>\n");
			printform();
		}
	}
}

/* Reads returned lines from waissearch, grabs the relevant information,
** and sticks it all in the return information structure.
*/

void read_from_waissearch(fp)
	FILE *fp;
{
	int i, sdocnum, score, lines, bytes, found;
	static char buffer[MAXSTRLEN], filename[MAXSTRLEN], title[MAXSTRLEN],
	type[MAXSTRLEN], icon[MAXSTRLEN];

	found = -1;
	while (fgets(buffer, MAXSTRLEN, fp) != NULL) {
		if (found == 0)
			return;
		if (version) {
			printf("%s\n", buffer);
			break;
		}
		if (strstr(buffer, "NumberOfRecordsReturned")) {
			for (i = 0; buffer[i] != ':'; i++)
				;
			for (found = 0; buffer[i]; i++)
				if (isdigit(buffer[i]))
					found = (found * 10) +
					(buffer[i] - '0');
			if (!found)
				return;
		}
		if (strstr(buffer, "Information on database") ||
		  strstr(buffer, "Catalog for database") ||
		  strstr(buffer, "Search produced no") ||
		  strstr(buffer, "is not available"))
			return;
		if (strstr(buffer, "View document") ||
		strstr(buffer, "q to quit"))
			return;
		if (strstr(buffer, "Score:")) {
			found--;
			for (i = sdocnum = 0; buffer[i] != ':'; i++)
				if (isdigit(buffer[i]))
					sdocnum = (sdocnum * 10) + (buffer[i] - '0');
			score = getnumber(buffer, 2);
			lines = getnumber(buffer, 3);
			strcpy(filename, (char *) getsfilename(buffer,
				sdocnum));
			bytes = getsize(filename);
			strcpy(type, (char *) gettype(filename));
			if (use_icons)
				strcpy(icon, (char *) geticon(filename));
			else
				strcpy(icon, NO_ICON);
			strcpy(title, (char *) gettitle(filename, sdocnum));
			entrylist = (struct entry *) addentry(entrylist,
			score, lines, bytes, filename, title, type, icon,
			sdocnum);
		}
	}
}
