
#include <stdio.h>
#include <ctype.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include "w4ais.h"


/* Grabs a number from a particular field in a line returned
** by waisq or waissearch, usually.
*/

int getnumber(line, field)
	char *line;
	int field;
{
	int i, j = 0, k;

	for (i = k = 0; k != field; k++) {
		for (j = 0; line[i] && !isdigit(line[i]); i++)
			;
		if (!line[i])
			return 0;
		while (isdigit(line[i]))
			j = (j * 10) + (line[i++] - '0');
	}

	return j;
}

/* Grabs filename information from waisq.
*/

char *getqfilename(line)
	char *line;
{
	int i, num, spaces;
	static char filename[MAXSTRLEN];
	char *c;

	c = (char *) strstr(line, "#(") + 2;
	for (i = num = spaces = 0; *c != '\0'; c++) {
		if (isdigit(*c)) {
			while (isdigit(*c)) {
				num = (num * 10) + (*c - '0');
				c++;
			}
			if (spaces >= 2)
				filename[i++] = num;
			if (num == 32)
				spaces++;
			num = 0;
		}
	}
	filename[i] = '\0';
	return filename;
}

/* Grabs filename information returned from waissearch. I don't
** like this very much...
*/

char *getsfilename(line, sdocnum)
	char *line;
	int sdocnum;
{
	int i;
	char *c;
	static char filename[MAXSTRLEN], wholeline[MAXSTRLEN],
	suffix[MAXSTRLEN], prefix[MAXSTRLEN];

	c = (char *) strchr(line, '\'') + 1;
	for (i = 0; *c && i < MAXSTRLEN && *c != '\''; c++)
		wholeline[i++] = *c;
	wholeline[i] = '\0';

	if ((!strchr(wholeline, '/') || !strchr(wholeline, '.')) ||
	sdocnum <= 0)
		return wholeline;

	c = (char *) strchr(line, '\'') + 1;
	for (i = 0; *c && i < MAXSTRLEN && *c != '\'' && !isspace(*c); c++)
		suffix[i++] = *c;
	suffix[i] = '\0';
	if (strstr(suffix, "://"))
		return suffix;

	while (*c && isspace(*c))
		c++;
	for (i = 0; *c && *c != '\'' && !isspace(*c) && i < MAXSTRLEN; c++)
		prefix[i++] = *c;
	prefix[i] = '\0';

	sprintf(filename, "%s%s", prefix, suffix);
	return filename;
}

/* Depending on the suffix, gets the type of file.
*/

char *gettype(filename)
	char *filename;
{
	struct suffixentry *tmplist;
	static char tmpsuffix[MAXSTRLEN], desc[MAXSTRLEN];

	if (!strchr(filename, '.'))
		return unknown_type;

	tmplist = suffixlist;
	while (tmplist != NULL) {
		strcpy(tmpsuffix, strrchr(filename, '.'));
		if (lstrstr(tmpsuffix, tmplist->suffix) &&
		strlen(tmpsuffix) == strlen(tmplist->suffix)) {
			strcpy(desc, tmplist->desc);
			return desc;
		}
		tmplist = tmplist->next;
	}

	return unknown_type;
}

/* Depending on the suffix, gets the right icon URL.
*/

char *geticon(filename)
	char *filename;
{
	struct suffixentry *tmplist;
	static char tmpsuffix[MAXSTRLEN], url[MAXSTRLEN];

	if (!strchr(filename, '.'))
		return unknown_icon;

	tmplist = suffixlist;
	while (tmplist != NULL) {
		strcpy(tmpsuffix, strrchr(filename, '.'));
		if (lstrstr(tmpsuffix, tmplist->suffix) &&
		strlen(tmpsuffix) == strlen(tmplist->suffix)) {
			strcpy(url, tmplist->url);
			return url;
		}
		tmplist = tmplist->next;
	}

	return unknown_icon;
}

/* Depending on the suffix, gets the right MIME type.
*/

char *getmime(filename)
	char *filename;
{
	struct suffixentry *tmplist;
	static char tmpsuffix[MAXSTRLEN], mime[MAXSTRLEN];

	if (!strchr(filename, '.'))
		return unknown_mime;

	tmplist = suffixlist;
	while (tmplist != NULL) {
		strcpy(tmpsuffix, strrchr(filename, '.'));
		if (lstrstr(tmpsuffix, tmplist->suffix) &&
		strlen(tmpsuffix) == strlen(tmplist->suffix)) {
			strcpy(mime, tmplist->mime);
			return mime;
		}
		tmplist = tmplist->next;
	}

	return unknown_mime;
}

/* From the returned filename, tries to extract a title of string of
** some sort for displaying on the page. This is the ugly part.
** If the file is a local HTML file, this extracts the title if
** it exists.
*/

char *gettitle(filename, sdocnum)
	char *filename;
	int sdocnum;
{
	int i, j;
	static char title[MAXTITLELEN], tmpline[MAXSTRLEN];
	FILE *fp;

	if (sdocnum > 0 && strstr(filename, "   ") &&
	strchr(filename, '/')) {
		for (i = j = 0; filename[i] && !isspace(filename[i]); i++)
			tmpline[j++] = filename[i];
		tmpline[j] = '\0';
		return tmpline;
	}
	if (strrchr(filename, '/') && filename[strlen(filename)] != '/')
		strcpy(tmpline, (char *) strrchr(filename, '/') + 1);
	else
		strcpy(tmpline, filename);
	if (!lstrstr(tmpline, ".html"))
		return tmpline;

	if ((fp = fopen(filename, "r")) == NULL)
		return tmpline;
	strcpy(title, parsetitle(fp));
	fclose(fp);
	if (strlen(title) > 0)
		return title;
	return tmpline;
}

/* What is the physical size of a file?
*/

int getsize(path)
	char *path;
{
	struct stat stbuf;

	if (stat(path, &stbuf))
		return -1;
	return stbuf.st_size;
}

/* From a line, grabs the next word and returns it. If the word is
** in quotes, spaces (and other characters) are counted as part of the word.
*/

char *getword(line, skiplen)
	char *line;
	int *skiplen;
{
	int i, inquotes;
	char *start;
	static char word[MAXSTRLEN];

	start = line;
	if (!(*line))
		return "\0";
	while (isspace(*line))
		line++;
	if (!(*line))
		return "\0";
	if (*line == '\"') {
		inquotes = 1;
		line++;
	}
	else
		inquotes = 0;
	for (i = 0; *line && i < MAXSTRLEN &&
	((inquotes) ? (*line != '\"') : (!isspace(*line))); line++)
		word[i++] = *line;
	word[i] = '\0';
	if (!(*line))
		return "\0";
	if (*line == '\"')
		line++;

	*skiplen = line - start;

	return word;
}
