/******************************************************************************
 *
 *  Package : w4ais 3.1 - swish/waisq/waissearch to HTML interface
 *  Date    : 02 Aug 1998
 *  Author  : Miles O'Neal
 *  Email   : meo@rru.com
 *
 *  LICENSE & COPYING
 *  -----------------
 *  w4ais is based on version 2.5 of wwwwais from EIT
 *  by Kevin Hughes.  At the time that w4ais work began,
 *  wwwwais was freely redistributable, and not under
 *  any explicit license.  As of September, 1997, wwwwais
 *  2.5 was GNU Copylefted.  Since w4ais predates September,
 *  1997, it is not subject to that license.  Instead, it
 *  is subject to the following Copyright & license.
 *
 *
 *  COPYRIGHT
 *
 *  Copyright 1995-1998 by Miles O'Neal, Austin, TX, USA.
 *
 *  All Rights Reserved, except as noted herein.
 *
 *  This software may be redistributed in any fashion you like, with only
 *  the following limitations.
 *
 *     1) You must credit the authors in the source code and accompanying
 *        documentation.
 *
 *     2) You may not use any of the names of the authors or their employers
 *        in any associated advertising without explicit permission from the
 *        parties you wish to name.  Finding and contacting them is up to you.
 *
 *     3) This copyright must be provided along with the documentation or
 *        code.
 *
 *     4) The accompanying disclaimer must must be provided along with the
 *        documentation or code.
 *
 *  If you can actually use this to make money, more power to you.  Just
 *  realize that the w4ais author is dedicated to keeping good, robust,
 *  useful code freely available to the public, and that philosophy applies
 *  to this package as well.
 *
 *
 *  WARRANTY & DISCLAIMER
 *
 *  This software is presented as is, with no warranties expressed
 *  or implied, including implied warranties of merchantability and
 *  fitness.  In no event shall the authors, their institutions, or
 *  any subsequent distributors be liable for any special, direct,
 *  indirect or consequential damages whatsoever resulting from loss
 *  of use, data or profits, whether in an action of contract,
 *  negligence or other tortious action, arising out of or in connection
 *  with the use or performance of this software. In other words, if
 *  you don't like it, don't use it!
 *
 *  Bugs and feature requests, complaints, and so forth, brought to the w4ais
 *  library author's attention, will be considered.  Any modifications will
 *  be made at the sole discretion of the author.
 *
 *
 *  [Non-obscene suggestions for improvement to this copyright and disclaimer
 *  are always welcome.  The intent is to keep control, simply so that nobody
 *  else takes control.  This document should be concise and user-friendly.]
 *
 *****************************************************************************/


#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define MAIN_MODULE 1
#include "w4ais.h"

static char *copyright = "\
Copyright 1995-1998 by Miles O'Neal, Austin, TX, USA.\n\
\n\
All Rights Reserved, except as noted herein.\n\
\n\
This software may be redistributed in any fashion you like, with only\n\
the following limitations.\n\
\n\
   1) You must credit the authors in the source code and accompanying\n\
      documentation.\n\
\n\
   2) You may not use any of the names of the authors or their employers\n\
      in any associated advertising without explicit permission from the\n\
      parties you wish to name.  Finding and contacting them is up to you.\n\
\n\
   3) This copyright must be provided along with the documentation or\n\
      code.\n\
\n\
   4) The accompanying disclaimer must must be provided along with the\n\
      documentation or code.\n\
\n\
If you can actually use this to make money, more power to you.  Just\n\
realize that the w4ais author is dedicated to keeping good, robust,\n\
useful code freely available to the public, and that philosophy applies\n\
to this package as well.\n\
";


int main()
{
	int i, len;
	char *p, *method, *query, *addr;

	fclose(stderr);

	sorttype[0] = '\0';
	selection[0] = '\0';
	srcdesc[0] = '\0';
	sourcelist = NULL;
	suffixlist = NULL;
	ruleslist = NULL;
	desclist = NULL;
	bodylist = NULL;
	use_icons = 0;
	use_selection = 0;
	indexsources = 0;
	docnum = -1;

	signal(SIGSEGV, badsegviol);
	signal(SIGPIPE, badconnection);
	signal(SIGALRM, badtimeout);

/* First, get all the normal environment CGI things...
*/
	if (NULL == copyright) exit(0); /* nuke last compiler warning 8^) */

	method = (char *) getenv("REQUEST_METHOD");
	if (method == NULL)
		progerr("Unknown method.");
	if (!strncmp(method, "POST", 4)) {
		len = atoi(getenv("CONTENT_LENGTH"));
		i = 0;
		while (len-- && i < MAXSTRLEN)
			query_string[i++] = fgetc(stdin);
		query_string[i] = '\0';
	}
	else if (!strncmp(method, "GET", 3)) {
		query = (char *) getenv("QUERY_STRING");
		if (query == NULL)
			query_string[0] = '\0';
		else
			strcpy(query_string, query);
	}
	else
		progerr("Unknown method.");

/* Does the user want version information?
** Is a document number associated with the request?
** Will the request be a WAIS source description?
*/

	version = ((getvalue(VERSIONTXT, ""))[0] == '\0') ? 0 : 1;
	docnum = atoi(getvalue(DOCNUMTXT, "-1"));
	strcpy(srcdesc, (char *) getvalue(DESCTXT, srcdesc));

/* Read the config file and determine any selected pop-up option...
*/

	strcpy(selection, (char *) getvalue(SELECTTXT, selection));
	if ((p = (char *) getenv("WWWW_SELECTION")) != NULL)
		strcpy(selection, p);
	if (selection[0] == '\0' || selection[0] == (char) NULL)
		use_selection = 0;
	if (!strcmp(selection, NO_SELECT))
		no_options = 1;
	else
		no_options = 0;

	strcpy(conffile, (char *) getvalue(CONFFILETXT, conffile));
	if ((p = (char *) getenv("WWWW_CONFFILE")) != NULL)
		strcpy(conffile, p);

	getdefaults();

/* Make sure the user's address is OK...
*/

	addr = (char *) getenv("REMOTE_ADDR");
	if (addr == NULL || !isokstring(addr, addrmask)) {
		printreject();
		exit(0);
	}

/* Grab all the normal variables.
*/

	strcpy(format, (char *) getvalue(FORMATTXT, format));
	if ((p = (char *) getenv("WWWW_FORMAT")) != NULL)
		strcpy(format, p);

	strcpy(maxhits, (char *) getvalue(MAXHITSTXT, maxhits));
	if ((p = (char *) getenv("WWWW_MAXHITS")) != NULL)
		strcpy(maxhits, p);

	strcpy(sorttype, (char *) getvalue(SORTTYPETXT, sorttype));
	if ((p = (char *) getenv("WWWW_SORTTYPE")) != NULL)
		strcpy(sorttype, p);

	strcpy(iconurl, (char *) getvalue(ICONURLTXT, iconurl));
	if ((p = (char *) getenv("WWWW_ICONURL")) != NULL)
		strcpy(iconurl, p);

	strcpy(useicons, (char *) getvalue(USEICONSTXT, useicons));
	if ((p = (char *) getenv("WWWW_USEICONS")) != NULL)
		strcpy(useicons, p);

	strcpy(keywords, (char *) getkeywords());
	if (lstrstr(keywords, KEYWORDSTXT))
		strcpy(keywords, "");
	if ((p = (char *) getenv("WWWW_KEYWORDS")) != NULL)
		strcpy(keywords, p);

/* If there is a pop-up selection, these variables (host, port, etc.)
** are overridden.
*/

	if (!use_selection) {
		strcpy(source, (char *) getvalue(SOURCETXT, source));
		if ((p = (char *) getenv("WWWW_SOURCE")) != NULL)
			strcpy(source, p);

		strcpy(sourcedir, (char *) getvalue(SOURCEDIRTXT, sourcedir));
		if ((p = (char *) getenv("WWWW_SOURCEDIR")) != NULL)
			strcpy(sourcedir, p);

		strcpy(host, (char *) getvalue(HOSTTXT, host));
		if ((p = (char *) getenv("WWWW_HOST")) != NULL)
			strcpy(host, p);

		strcpy(port, (char *) getvalue(PORTTXT, port));
		if ((p = (char *) getenv("WWWW_PORT")) != NULL)
			strcpy(port, p);

		strcpy(searchprog,
			(char *) getvalue(SEARCHPROGTXT, searchprog));
		if ((p = (char *) getenv("WWWW_SEARCHPROG")) != NULL)
			strcpy(searchprog, p);
	}

	entrylist = NULL;

	if (lstrstr(useicons, "yes"))
		use_icons = 1;
	else
		use_icons = 0;

/* Do any searching that needs to be done.
** If there are no keywords, just print a form.
*/

	if (srcdesc[0] != '\0') {
		alarm(TIMEOUT);
		descwaiss();
	} else if (!keywords[0] && !version) {
		printheader();
		printform();
		printfooter();
		return 0;
	} else {
		if (version) {
			printversion();
			if (isfile(waisqbin))
				dowaisq();
			if (isfile(waissearchbin))
				dowaiss();
			if (isfile(swishbin))
				doswish();
		} else {
			alarm(TIMEOUT);
			if (!strcmp(searchprog, "waisq"))
				dowaisq();
			else if (!strcmp(searchprog, "waissearch"))
				dowaiss();
			else
				doswish();
			printfooter();
		}
	}

	return 0;
}
